@@ -22,9 +22,20 @@ GNU General Public License for more details.
 
 #include <openbabel/parsmart.h>
 #include <openbabel/typer.h>
+#include <openbabel/bitvec.h>
+
+#ifdef UNUSED
+#elif (__GNUC__ == 4)
+# define UNUSED(x) UNUSED_ ## x __attribute__((unused))
+#elif defined(__LCLINT__)
+# define UNUSED(x) /*@unused@*/ x
+#else
+# define UNUSED(x) x
+#endif
 
 namespace OpenBabel
 {
+  class OBRing;
 
 #ifndef SQUARE
 #define SQUARE(x) ((x)*(x))
@@ -153,13 +164,13 @@ namespace OpenBabel
     void SetBond(OBBond *bond)
     {
       _bond = bond;
-      SetRings(bond);
+      SetRings();
     }
     /**
      * Set the rings associated with this bond (if it's a ring bond)
      * \since Version 2.4
      */
-    void SetRings(OBBond *bond);
+    void SetRings();
     /**
      * Set the index for this rotor. Used by OBRotorList
      */
@@ -489,7 +500,7 @@ namespace OpenBabel
      */
     bool HasFixedBonds()
     {
-      return !_fixedbonds.Empty();
+      return !_fixedbonds.IsEmpty();
     }
     //! Rotates each bond to zero and 180 degrees and tests
     //! if the 2 conformers are duplicates.  if so - the symmetric torsion
@@ -618,7 +629,7 @@ namespace OpenBabel
      */
     bool HasFixedAtoms()
     {
-      return(!_fixedatoms.Empty());
+      return(!_fixedatoms.IsEmpty());
     }
     //! Has no effect
     //! \deprecated Currently has no effect
