@@ -20,11 +20,18 @@ GNU General Public License for more details.
 #include <openbabel/babelconfig.h>
 
 #include <openbabel/atom.h>
-#include <openbabel/stereo/stereo.h>
+#include <openbabel/bond.h>
 #include <openbabel/mol.h>
+#include <openbabel/obiter.h>
 #include <openbabel/molchrg.h>
+#include <openbabel/ring.h>
 #include <openbabel/phmodel.h>
 #include <openbabel/builder.h>
+#include <openbabel/elements.h>
+#include <openbabel/chains.h>
+#include <openbabel/obutil.h>
+#include <openbabel/residue.h>
+#include <openbabel/chains.h>
 
 #include <openbabel/math/matrix3x3.h>
 
@@ -34,9 +41,10 @@ extern "C" int strncasecmp(const char *s1, const char *s2, size_t n);
 
 using namespace std;
 
+
 namespace OpenBabel
 {
-
+  EXTERN OBChainsParser chainsparser;
   /** \class OBAtom atom.h <openbabel/atom.h>
       \brief Atom class
 
@@ -95,10 +103,11 @@ namespace OpenBabel
       \endcode
   */
 
-  extern OBAromaticTyper  aromtyper;
-  extern OBAtomTyper      atomtyper;
-  extern OBPhModel        phmodel;
-
+  extern THREAD_LOCAL OBAromaticTyper  aromtyper;
+  extern THREAD_LOCAL OBAtomTyper      atomtyper;
+  extern THREAD_LOCAL OBPhModel        phmodel;
+  EXTERN OBTypeTable      ttab;
+  
   //
   // OBAtom member functions
   //
@@ -136,7 +145,7 @@ namespace OpenBabel
     _ele = (char)0;
     _isotope = 0;
     _spinmultiplicity=0; // CM 18 Sept 2003
-    _impval = 0;
+    _imph = 0;
     _fcharge = 0;
     _type[0] = '\0';
     _pcharge = 0.0;
@@ -164,18 +173,19 @@ namespace OpenBabel
     if (!src)
       return;
 
-    _hyb = src->GetHyb();
-    _ele = src->GetAtomicNum();
-    _isotope = src->GetIsotope();
-    _fcharge = src->GetFormalCharge();
-    _spinmultiplicity = src->GetSpinMultiplicity();
-    strncpy(_type,src->GetType(), sizeof(_type) - 1);
+    _hyb = src->_hyb;
+    _ele = src->_ele;
+    _imph = src->_imph;
+    _isotope = src->_isotope;
+    _fcharge = src->_fcharge;
+    _spinmultiplicity = src->_spinmultiplicity;
+    strncpy(_type,src->_type, sizeof(_type) - 1);
     _type[sizeof(_type) - 1] = '\0';
-    _pcharge = src->GetPartialCharge();
+    _pcharge = src->_pcharge;
     _v = src->GetVector();
-    _flags = src->GetFlag();
+    _flags = src->_flags;
     _residue = (OBResidue*)NULL;
-    _id = src->GetId();
+    _id = src->_id;
 
     _vdata.clear();
     //Copy all the OBGenericData, providing the new atom
@@ -266,9 +276,9 @@ namespace OpenBabel
     OBBondIterator i,j;
 
     for (a1 = BeginNbrAtom(i);a1;a1 = NextNbrAtom(i))
-      if (includePandS || (!a1->IsPhosphorus() && !a1->IsSulfur()))
+      if (includePandS || (a1->GetAtomicNum() != OBElements::Phosphorus && a1->GetAtomicNum() != OBElements::Sulfur))
         for (a2 = a1->BeginNbrAtom(j);a2;a2 = a1->NextNbrAtom(j))
-          if (a2 != this && ((*j)->GetBO() == 2 || (*j)->GetBO() == 3 || (*j)->GetBO() == 5))
+          if (a2 != this && ((*j)->GetBondOrder() == 2 || (*j)->GetBondOrder() == 3 || (*j)->GetBondOrder() == 5))
             return(true);
 
     return(false);
@@ -279,7 +289,7 @@ namespace OpenBabel
     OBBond *bond;
     OBBondIterator i;
     for (bond = BeginBond(i);bond;bond = NextBond(i))
-      if (bond->GetBO() == order)
+      if (bond->GetBondOrder() == order)
         return(true);
 
     return(false);
@@ -291,7 +301,7 @@ namespace OpenBabel
     OBBond *bond;
     OBBondIterator i;
     for (bond = BeginBond(i);bond;bond = NextBond(i))
-      if (bond->GetBO() == order)
+      if (bond->GetBondOrder() == order)
         count++;
 
     return(count);
@@ -299,12 +309,12 @@ namespace OpenBabel
 
   int OBAtom::HighestBondOrder()
   {
-    int highest = 0;
+    unsigned int highest = 0;
     OBBond *bond;
     OBBondIterator i;
     for(bond = BeginBond(i); bond; bond = NextBond(i))
-      if(bond->GetBO() > highest)
-        highest = bond->GetBO();
+      if(bond->GetBondOrder() > highest)
+        highest = bond->GetBondOrder();
 
     return(highest);
   }
@@ -314,7 +324,7 @@ namespace OpenBabel
     OBBond *bond;
     OBBondIterator i;
     for (bond = BeginBond(i);bond;bond = NextBond(i))
-      if (bond->GetBO() != 1)
+      if (bond->GetBondOrder() != 1)
         return(true);
 
     return(false);
@@ -322,7 +332,7 @@ namespace OpenBabel
 
   bool OBAtom::IsPolarHydrogen()
   {
-    if (!IsHydrogen())
+    if (GetAtomicNum() != OBElements::Hydrogen)
       return(false);
 
     OBAtom *atom;
@@ -346,7 +356,7 @@ namespace OpenBabel
 
   bool OBAtom::IsNonPolarHydrogen()
   {
-    if (!IsHydrogen())
+    if (GetAtomicNum() != OBElements::Hydrogen)
       return(false);
 
     OBAtom *atom;
@@ -429,59 +439,29 @@ namespace OpenBabel
 
   void OBAtom::SetIsotope(unsigned int iso)
   {
-    if (_ele == 1 && iso == 2)
-      SetType("D");
-    else if (_ele == 1 && (iso == 1 || iso == 0))
-      SetType("H");
-
     _isotope = iso;
   }
 
-  OBAtom *OBAtom::GetNextAtom()
-  {
-    OBMol *mol = (OBMol*)GetParent();
-    return(((unsigned)GetIdx() == mol->NumAtoms())? NULL : mol->GetAtom(GetIdx()+1));
-  }
-
   OBResidue *OBAtom::GetResidue()
   {
-    return GetResidue(true); // default is to always perceive chains
-  }
+    OBMol *mol = this->GetParent();
+    if (!mol->HasChainsPerceived())
+      chainsparser.PerceiveChains(*mol);
 
-  OBResidue *OBAtom::GetResidue(bool perception)
-  {
-    if (_residue != NULL)
-      return _residue;
-    else if (perception && !((OBMol*)GetParent())->HasChainsPerceived())
-      {
-        ((OBMol*)GetParent())->SetChainsPerceived();
-        if ( chainsparser.PerceiveChains(*((OBMol*)GetParent())) )
-          return _residue;
-        else
-          {
-            if (_residue)
-              {
-                delete _residue;
-                _residue = NULL;
-              }
-            return NULL;
-          }
-      }
-    else
-      return NULL;
+    return _residue;
   }
 
   double OBAtom::GetAtomicMass() const
   {
     if (_isotope == 0)
-      return etab.GetMass(_ele);
+      return OBElements::GetMass(_ele);
     else
-      return isotab.GetExactMass(_ele, _isotope);
+      return OBElements::GetExactMass(_ele, _isotope);
   }
 
   double OBAtom::GetExactMass() const
   {
-    return isotab.GetExactMass(_ele, _isotope);
+    return OBElements::GetExactMass(_ele, _isotope);
   }
 
   char *OBAtom::GetType()
@@ -511,22 +491,6 @@ namespace OpenBabel
     return(_type);
   }
 
-  unsigned int OBAtom::GetImplicitValence() const
-  {
-    //Special case for [H] to avoid infite loop: SMARTS Match() <-> AssignSpinMultiplicity()
-    if(GetAtomicNum() == 1) {
-      unsigned int val = GetValence();
-      if (val == 0 && GetFormalCharge() == 0 && GetSpinMultiplicity() == 0)
-        return 1;
-      return val;
-    }
-    OBMol *mol = (OBMol*)((OBAtom*)this)->GetParent();
-    if (mol && !mol->HasImplicitValencePerceived())
-      atomtyper.AssignImplicitValence(*((OBMol*)((OBAtom*)this)->GetParent()));
-
-    return((unsigned int)_impval);
-  }
-
   unsigned int OBAtom::GetHyb() const
   {
     //hybridization is assigned when atoms are typed
@@ -538,20 +502,20 @@ namespace OpenBabel
   }
 
 
-  unsigned int OBAtom::GetHvyValence() const
+  unsigned int OBAtom::GetHvyDegree() const
   {
     unsigned int count=0;
 
     OBBond *bond;
     OBBondIterator i;
     for (bond = ((OBAtom*)this)->BeginBond(i); bond; bond = ((OBAtom*)this)->NextBond(i))
-      if (!(bond->GetNbrAtom((OBAtom*)this)->IsHydrogen()))
+      if (bond->GetNbrAtom((OBAtom*)this)->GetAtomicNum() != OBElements::Hydrogen)
         count++;
 
     return(count);
   }
 
-  unsigned int OBAtom::GetHeteroValence() const
+  unsigned int OBAtom::GetHeteroDegree() const
   {
     unsigned int count=0;
     OBBond *bond;
@@ -590,7 +554,7 @@ namespace OpenBabel
   //! Returns true if nitrogen is part of an amide
   bool OBAtom::IsAmideNitrogen()
   {
-    if (!IsNitrogen())
+    if (GetAtomicNum() != OBElements::Nitrogen)
       return(false);
 
     OBAtom *nbratom,*atom;
@@ -602,7 +566,7 @@ namespace OpenBabel
       {
         nbratom = bond->GetNbrAtom(atom);
         for (abbond = nbratom->BeginBond(j);abbond;abbond = nbratom->NextBond(j))
-          if (abbond->GetBO() == 2 &&
+          if (abbond->GetBondOrder() == 2 &&
               (((abbond->GetNbrAtom(nbratom))->GetAtomicNum() == 8) ||
                ((abbond->GetNbrAtom(nbratom))->GetAtomicNum() == 16)))
             return(true);
@@ -613,14 +577,14 @@ namespace OpenBabel
 
   bool OBAtom::IsAromaticNOxide()
   {
-    if (!IsNitrogen() || !IsAromatic())
+    if (GetAtomicNum() != OBElements::Nitrogen || !IsAromatic())
       return(false);
 
     OBAtom *atom;
     OBBondIterator i;
 
     for (atom = BeginNbrAtom(i);atom;atom = NextNbrAtom(i))
-      if (atom->IsOxygen() && !(*i)->IsInRing() && (*i)->GetBO() == 2)
+      if (atom->GetAtomicNum() == OBElements::Oxygen && !(*i)->IsInRing() && (*i)->GetBondOrder() == 2)
         return(true);
 
     return(false);
@@ -628,9 +592,9 @@ namespace OpenBabel
 
   bool OBAtom::IsCarboxylOxygen()
   {
-    if (!IsOxygen())
+    if (GetAtomicNum() != OBElements::Oxygen)
       return(false);
-    if (GetHvyValence() != 1)
+    if (GetHvyDegree() != 1)
       return(false);
 
     OBAtom *atom;
@@ -639,7 +603,7 @@ namespace OpenBabel
 
     atom = NULL;
     for (bond = BeginBond(i);bond;bond = NextBond(i))
-      if ((bond->GetNbrAtom(this))->IsCarbon())
+      if ((bond->GetNbrAtom(this))->GetAtomicNum() == OBElements::Carbon)
         {
           atom = bond->GetNbrAtom(this);
           break;
@@ -657,9 +621,9 @@ namespace OpenBabel
 
   bool OBAtom::IsPhosphateOxygen()
   {
-    if (!IsOxygen())
+    if (GetAtomicNum() != OBElements::Oxygen)
       return(false);
-    if (GetHvyValence() != 1)
+    if (GetHvyDegree() != 1)
       return(false);
     OBAtom *atom;
     OBBond *bond;
@@ -667,7 +631,7 @@ namespace OpenBabel
 
     atom = NULL;
     for (bond = BeginBond(i);bond;bond = NextBond(i))
-      if ((bond->GetNbrAtom(this))->IsPhosphorus())
+      if ((bond->GetNbrAtom(this))->GetAtomicNum() == OBElements::Phosphorus)
         {
           atom = bond->GetNbrAtom(this);
           break;
@@ -684,9 +648,9 @@ namespace OpenBabel
 
   bool OBAtom::IsSulfateOxygen()
   {
-    if (!IsOxygen())
+    if (GetAtomicNum() != OBElements::Oxygen)
       return(false);
-    if (GetHvyValence() != 1)
+    if (GetHvyDegree() != 1)
       return(false);
 
     OBAtom *atom;
@@ -695,7 +659,7 @@ namespace OpenBabel
 
     atom = NULL;
     for (bond = BeginBond(i);bond;bond = NextBond(i))
-      if ((bond->GetNbrAtom(this))->IsSulfur())
+      if ((bond->GetNbrAtom(this))->GetAtomicNum() == OBElements::Sulfur)
         {
           atom = bond->GetNbrAtom(this);
           break;
@@ -718,9 +682,9 @@ namespace OpenBabel
   //e.g. C-SO2-C
   // Is this atom an oxygen in a sulfone(R1 - SO2 - R2) group ?
   {
-    if (!atm->IsOxygen())
+    if (atm->GetAtomicNum() != OBElements::Oxygen)
       return(false);
-    if (atm->GetHvyValence() != 1){
+    if (atm->GetHvyDegree() != 1){
       //cerr << "sulfone> O valence is not 1\n";
       return(false);
       }
@@ -731,7 +695,7 @@ namespace OpenBabel
 
     // searching for attached sulfur
     for (bond1 = atm->BeginBond(i); bond1; bond1 = atm->NextBond(i))
-      if ((bond1->GetNbrAtom(atm))->IsSulfur())
+      if ((bond1->GetNbrAtom(atm))->GetAtomicNum() == OBElements::Sulfur)
         { nbr = bond1->GetNbrAtom(atm);
           break; }
     if (!nbr){
@@ -739,7 +703,7 @@ namespace OpenBabel
       return(false); }
 
     // check for sulfate
-    //cerr << "sulfone> If we're here... " << atom->GetAtomicNum() <<"\n" << atom->IsSulfur() << "\n";
+    //cerr << "sulfone> If we're here... " << atom->GetAtomicNum() <<"\n" << atom->GetAtomicNum() == OBElements::Sulfur << "\n";
     //cerr << "sulfone> number of free oxygens:" << atom->CountFreeOxygens() << "\n";
     if (nbr->CountFreeOxygens() != 2){
       //cerr << "sulfone> count of free oxygens not 2" << atom->CountFreeOxygens() << '\n' ;
@@ -748,7 +712,7 @@ namespace OpenBabel
     // check for sulfonamide
     for (bond2 = nbr->BeginBond(j);bond2;bond2 = nbr->NextBond(j)){
       //cerr<<"NEIGH: " << (bond2->GetNbrAtom(atom))->GetAtomicNum()<<"\n";
-      if ((bond2->GetNbrAtom(nbr))->IsNitrogen()){
+      if ((bond2->GetNbrAtom(nbr))->GetAtomicNum() == OBElements::Nitrogen){
         //cerr << "sulfone> sulfonamide null\n" ;
         return(false);}}
     //cerr << "sulfone> none of the above\n";
@@ -761,9 +725,9 @@ namespace OpenBabel
 
   bool OBAtom::IsNitroOxygen()
   {
-    if (!IsOxygen())
+    if (GetAtomicNum() != OBElements::Oxygen)
       return(false);
-    if (GetHvyValence() != 1)
+    if (GetHvyDegree() != 1)
       return(false);
 
     OBAtom *atom;
@@ -772,7 +736,7 @@ namespace OpenBabel
 
     atom = NULL;
     for (bond = BeginBond(i);bond;bond = NextBond(i))
-      if ((bond->GetNbrAtom(this))->IsNitrogen())
+      if ((bond->GetNbrAtom(this))->GetAtomicNum() == OBElements::Nitrogen)
         {
           atom = bond->GetNbrAtom(this);
           break;
@@ -806,48 +770,28 @@ namespace OpenBabel
     return(false);
   }
 
-  bool OBAtom::IsNotCorH()
-  {
-    switch(GetAtomicNum())
-      {
-      case 1:
-      case 6:
-        return(false);
-      }
-    return(true);
-  }
-
   bool OBAtom::IsAromatic() const
   {
-    if (((OBAtom*)this)->HasFlag(OB_AROMATIC_ATOM))
-      return(true);
-
-    OBMol	*mol = (OBMol*)((OBAtom*)this)->GetParent();
-
+    OBMol *mol = ((OBAtom*)this)->GetParent();
     if (!mol->HasAromaticPerceived())
-      {
-        aromtyper.AssignAromaticFlags(*mol);
-        if (((OBAtom*)this)->HasFlag(OB_AROMATIC_ATOM))
-          return(true);
-      }
+      aromtyper.AssignAromaticFlags(*mol);
 
-    return(false);
+    if (((OBAtom*)this)->HasFlag(OB_AROMATIC_ATOM))
+      return true;
+
+    return false;
   }
 
   bool OBAtom::IsInRing() const
   {
-    if (((OBAtom*)this)->HasFlag(OB_RING_ATOM))
-      return(true);
-
-    OBMol *mol = (OBMol*)((OBAtom*)this)->GetParent();
+    OBMol *mol = ((OBAtom*)this)->GetParent();
     if (!mol->HasRingAtomsAndBondsPerceived())
-      {
-        mol->FindRingAtomsAndBonds();
-        if (((OBAtom*)this)->HasFlag(OB_RING_ATOM))
-          return(true);
-      }
+      mol->FindRingAtomsAndBonds();
 
-    return(false);
+    if (((OBAtom*)this)->HasFlag(OB_RING_ATOM))
+      return true;
+
+    return false;
   }
 
   //! @todo
@@ -1004,7 +948,7 @@ namespace OpenBabel
     for (bond = ((OBAtom*)this)->BeginBond(i);bond;bond = ((OBAtom*)this)->NextBond(i))
       {
         atom = bond->GetNbrAtom((OBAtom*)this);
-        if (atom->IsOxygen() && atom->GetHvyValence() == 1)
+        if (atom->GetAtomicNum() == OBElements::Oxygen && atom->GetHvyDegree() == 1)
           count++;
       }
 
@@ -1021,68 +965,27 @@ namespace OpenBabel
     for (bond = ((OBAtom*)this)->BeginBond(i);bond;bond = ((OBAtom*)this)->NextBond(i))
       {
         atom = bond->GetNbrAtom((OBAtom*)this);
-        if (atom->IsSulfur() && atom->GetHvyValence() == 1)
+        if (atom->GetAtomicNum() == OBElements::Sulfur && atom->GetHvyDegree() == 1)
           count++;
       }
 
     return(count);
   }
 
-  unsigned int OBAtom::BOSum() const
+  unsigned int OBAtom::GetExplicitValence() const
   {
-    unsigned int bo;
-    unsigned int bosum=0;
-    OBBond *bond;
+    unsigned int bosum = 0;
+    
     OBBondIterator i;
+    for (OBBond *bond = ((OBAtom*)this)->BeginBond(i); bond; bond = ((OBAtom*)this)->NextBond(i))
+      bosum += bond->GetBondOrder();
 
-    for (bond = ((OBAtom*)this)->BeginBond(i);bond;bond = ((OBAtom*)this)->NextBond(i))
-      {
-        bo = bond->GetBO();
-        bosum += (bo < 5) ? 2*bo : 3;
-      }
-
-    bosum /= 2;
-    return(bosum);
+    return bosum;
   }
 
-  unsigned int OBAtom::KBOSum() const
+  unsigned int OBAtom::GetTotalValence() const
   {
-    OBBond *bond;
-    unsigned int bosum;
-    OBBondIterator i;
-
-    bosum = GetImplicitValence();
-
-    for (bond = ((OBAtom*)this)->BeginBond(i);bond;bond = ((OBAtom*)this)->NextBond(i))
-      {
-        if (bond->IsKDouble())
-          bosum++;
-        else if (bond->IsKTriple())
-          bosum += 2;
-      }
-
-    return(bosum);
-  }
-
-  unsigned int OBAtom::ImplicitHydrogenCount() const
-  {
-    //handles H,C,N,S,O,X
-    OBMol *mol = (OBMol*)((OBAtom*)this)->GetParent();
-    if (mol && !mol->HasImplicitValencePerceived())
-      atomtyper.AssignImplicitValence(*((OBMol*)((OBAtom*)this)->GetParent()));
-
-    // _impval is assigned by the atomtyper -- same as calling GetImplicitValence()
-    int impval = _impval - GetValence();
-
-    // we need to modify this implicit valence if we're a radical center
-    int mult = GetSpinMultiplicity();
-    if(mult==2) //radical
-      impval-=1;
-    else if(mult==1 || mult==3) //carbene
-      impval-=2;
-    else if(mult>=4) //CH, Catom
-      impval -= mult-1;
-    return((impval>0)?impval:0);
+    return GetExplicitValence() + _imph;
   }
 
   unsigned int OBAtom::ExplicitHydrogenCount(bool ExcludeIsotopes) const
@@ -1093,7 +996,7 @@ namespace OpenBabel
     OBAtom *atom;
     OBBondIterator i;
     for (atom = ((OBAtom*)this)->BeginNbrAtom(i);atom;atom = ((OBAtom*)this)->NextNbrAtom(i))
-      if (atom->IsHydrogen() && !(ExcludeIsotopes && atom->GetIsotope()!=0))
+      if (atom->GetAtomicNum() == OBElements::Hydrogen && !(ExcludeIsotopes && atom->GetIsotope()!=0))
         numH++;
 
     return(numH);
@@ -1135,7 +1038,7 @@ namespace OpenBabel
       int S = SHELL[N];
       int V = VALENCE[N];
       int C = GetFormalCharge();
-      int B = ImplicitHydrogenCount() + BOSum();
+      int B = GetTotalValence();
       // TODO: Do we actually want to divide by 2 here? (counting pairs instead of single)
       counts.first = (S - V - B + C) / 2;  // Acid: Number of electrons pairs desired
       counts.second = (V - B - C) / 2;     // Base: Number of electrons pairs available
@@ -1249,9 +1152,24 @@ namespace OpenBabel
     return(true);
   }
 
+  //! \return a "corrected" bonding radius based on the hybridization.
+  //! Scales the covalent radius by 0.95 for sp2 and 0.90 for sp hybrids
+  static double CorrectedBondRad(unsigned int elem, unsigned int hyb)
+  {
+    double rad = OBElements::GetCovalentRad(elem);
+    switch (hyb) {
+    case 2:
+      return rad * 0.95;
+    case 1:
+      return rad * 0.90;
+    default:
+      return rad;
+    }
+  }
+
   bool OBAtom::HtoMethyl()
   {
-    if (!IsHydrogen())
+    if (GetAtomicNum() != OBElements::Hydrogen)
       return(false);
 
     obErrorLog.ThrowError(__FUNCTION__,
@@ -1277,12 +1195,12 @@ namespace OpenBabel
       }
 
     double br1,br2;
-    br1 = etab.CorrectedBondRad(6,3);
-    br2 = etab.CorrectedBondRad(atom->GetAtomicNum(),atom->GetHyb());
+    br1 = CorrectedBondRad(6,3);
+    br2 = CorrectedBondRad(atom->GetAtomicNum(),atom->GetHyb());
     bond->SetLength(atom,br1+br2);
 
     OBAtom *hatom;
-    br2 = etab.CorrectedBondRad(1,0);
+    br2 = CorrectedBondRad(1,0);
     vector3 v;
 
     for (int j = 0;j < 3;++j)
@@ -1329,13 +1247,13 @@ namespace OpenBabel
     //if (hyb == GetHyb()) return(true);
     if (GetAtomicNum() == 1)
       return(false);
-    if (hyb == 0 && GetHvyValence() > 1)
+    if (hyb == 0 && GetHvyDegree() > 1)
       return(false);
-    if (hyb == 1 && GetHvyValence() > 2)
+    if (hyb == 1 && GetHvyDegree() > 2)
       return(false);
-    if (hyb == 2 && GetHvyValence() > 3)
+    if (hyb == 2 && GetHvyDegree() > 3)
       return(false);
-    if (hyb == 3 && GetHvyValence() > 4)
+    if (hyb == 3 && GetHvyDegree() > 4)
       return(false);
 
     OBMol *mol = (OBMol*)GetParent();
@@ -1345,7 +1263,7 @@ namespace OpenBabel
     OBBondIterator i;
 
     for (nbr = BeginNbrAtom(i);nbr;nbr = NextNbrAtom(i))
-      if (nbr->IsHydrogen())
+      if (nbr->GetAtomicNum() == OBElements::Hydrogen)
         delatm.push_back(nbr);
 
     //delete attached hydrogens
@@ -1370,22 +1288,22 @@ namespace OpenBabel
 
     //adjust attached acyclic bond lengths
     double br1,br2, length;
-    br1 = etab.CorrectedBondRad(GetAtomicNum(),hyb);
+    br1 = CorrectedBondRad(GetAtomicNum(),hyb);
     for (nbr = BeginNbrAtom(i);nbr;nbr = NextNbrAtom(i))
       if (!(*i)->IsInRing())
         {
-          br2 = etab.CorrectedBondRad(nbr->GetAtomicNum(),nbr->GetHyb());
+          br2 = CorrectedBondRad(nbr->GetAtomicNum(),nbr->GetHyb());
           length = br1 + br2;
           if ((*i)->IsAromatic())
             length *= 0.93;
-          else if ((*i)->GetBO() == 2)
+          else if ((*i)->GetBondOrder() == 2)
             length *= 0.91;
-          else if ((*i)->GetBO() == 3)
+          else if ((*i)->GetBondOrder() == 3)
             length *= 0.87;
           ((OBBond*) *i)->SetLength(this, length);
         }
 
-    if (GetValence() > 1)
+    if (GetExplicitDegree() > 1)
       {
         double angle;
         matrix3x3 m;
@@ -1636,14 +1554,14 @@ namespace OpenBabel
         impval = 1;
       }
 
-    int hcount = impval-GetHvyValence();
+    int hcount = impval-GetHvyDegree();
     if (hcount)
       {
         int k;
         vector3 v;
         OBAtom *atom;
-        double brsum = etab.CorrectedBondRad(1,0)+
-          etab.CorrectedBondRad(GetAtomicNum(),GetHyb());
+        double brsum = CorrectedBondRad(1,0)+
+          CorrectedBondRad(GetAtomicNum(),GetHyb());
         SetHyb(hyb);
 
         mol->BeginModify();
@@ -1808,8 +1726,8 @@ namespace OpenBabel
       return true;
     if (_ele == 7) {
       // N+ ions and sp2 hybrid N with 3 valences should not be Hbond acceptors
-      if (!((GetValence() == 4 && GetHyb() == 3)
-            || (GetValence() == 3 && GetHyb() == 2)))
+      if (!((GetExplicitDegree() == 4 && GetHyb() == 3)
+            || (GetExplicitDegree() == 3 && GetHyb() == 2)))
             return true;
     }
     // Changes from Paolo Tosco
@@ -1875,7 +1793,7 @@ namespace OpenBabel
             }
           }
           else { 
-            if (nbr->IsHydrogen()) { // hydroxyl (YES)
+            if (nbr->GetAtomicNum() == OBElements::Hydrogen) { // hydroxyl (YES)
               return(true); 
             }
             else {
@@ -1899,8 +1817,8 @@ namespace OpenBabel
     };
     if (_ele == 7) {
       // N+ ions and sp2 hybrid N with 3 valences should not be Hbond acceptors
-      if (!((GetValence() == 4 && GetHyb() == 3)
-        || (GetValence() == 3 && GetHyb() == 2)))
+      if (!((GetExplicitDegree() == 4 && GetHyb() == 3)
+        || (GetExplicitDegree() == 3 && GetHyb() == 2)))
         return true;
     };
     // Changes from Paolo Tosco
@@ -1919,7 +1837,7 @@ namespace OpenBabel
     OBAtom *nbr;
     OBBondIterator i;
     for (nbr = BeginNbrAtom(i);nbr;nbr = NextNbrAtom(i))
-      if (nbr->IsHydrogen())
+      if (nbr->GetAtomicNum() == OBElements::Hydrogen)
         return true;
 
     return false;
@@ -1927,7 +1845,7 @@ namespace OpenBabel
 
   bool OBAtom::IsHbondDonorH()
   {
-    if (!IsHydrogen()) return(false);
+    if (GetAtomicNum() != OBElements::Hydrogen) return(false);
 
     // Recursive definition -- this atom is an H atom
     // are any neighbors HbondDonors?
