@@ -2,6 +2,7 @@
 conformersearch.cpp - Conformer searching using genetic algorithm.
 
 Copyright (C) 2010 Tim Vandermeersch
+Some portions Copyright (C) 2016 Torsten Sachse
 
 This file is part of the Open Babel project.
 For more information, see <http://openbabel.org/>
@@ -19,6 +20,15 @@ GNU General Public License for more details.
 #include <openbabel/conformersearch.h>
 #include <openbabel/math/align.h>
 #include <openbabel/forcefield.h>
+#include <openbabel/elements.h>
+#include <openbabel/bond.h>
+#include "rand.h"
+
+#if defined(_MSC_VER) && (_MSC_VER < 1800)
+ #define OB_ISNAN _isnan
+#else
+ #define OB_ISNAN std::isnan
+#endif
 
 namespace OpenBabel {
 
@@ -57,7 +67,7 @@ namespace OpenBabel {
         atom1 = mol.GetAtom(a1+1);
         atom2 = mol.GetAtom(a2+1);
         // Default should be to recognize H clashes too
-        if (!m_check_hydrogens  && (atom1->IsHydrogen() || atom2->IsHydrogen() ))
+        if (!m_check_hydrogens  && (atom1->GetAtomicNum() == OBElements::Hydrogen || atom2->GetAtomicNum() == OBElements::Hydrogen ))
           continue;
 
         // skip connected atoms
@@ -70,8 +80,8 @@ namespace OpenBabel {
         distanceSquared = dx*dx + dy*dy + dz*dz;
         // As we don't check 1-3 and 1-4 bonded atoms, apply a
         // factor of to the sum of VdW radii
-        vdwCutoff = m_vdw_factor * (etab.GetVdwRad(atom1->GetAtomicNum())
-                                    + etab.GetVdwRad(atom2->GetAtomicNum()));
+        vdwCutoff = m_vdw_factor * (OBElements::GetVdwRad(atom1->GetAtomicNum())
+                                    + OBElements::GetVdwRad(atom2->GetAtomicNum()));
         vdwCutoff *= vdwCutoff; // compare squared distances
         //cout << vdwCutoff << " " << m_vdw_factor << " " << m_cutoff << " " <<  distanceSquared << endl ;
 
@@ -299,15 +309,22 @@ namespace OpenBabel {
     p_crossover = 0.7;
     niche_mating = 0.7;
     local_opt_rate = 3;
-    unique_generator.TimeSeed();
+    // For the moment 'd' is an opaque pointer to an instance of OBRandom*.
+    // In future, it could be a pointer to a structure storing all of the
+    // private variables.
+    d = (void*)new OBRandom();
+    ((OBRandom*)d)->TimeSeed();
     m_logstream = &cout; 	// Default logging send to standard output
     // m_logstream = NULL;
+    m_printrotors = false;  // By default, do not print rotors but perform the conformer search
+
   }
 
   OBConformerSearch::~OBConformerSearch()
   {
     delete m_filter;
     delete m_score;
+    delete (OBRandom*)d;
   }
 
 
@@ -327,6 +344,46 @@ namespace OpenBabel {
     // Initialize the OBRotorList
     m_rotorList.SetFixedBonds(m_fixedBonds);
     m_rotorList.Setup(m_mol);
+
+    // Print all available rotors if so desired
+    if (m_printrotors){
+      OBRotorIterator it;
+      OBRotorIterator end_it = m_rotorList.EndRotors();
+      OBRotor* r = m_rotorList.BeginRotor(it);
+      int rotcount = 1;
+      std::cout << "Rotors:" << std::endl;
+      while(r){
+        OBBond* b = r->GetBond();
+        int at1,at2;
+        at1 = b->GetBeginAtomIdx();
+        at2 = b->GetEndAtomIdx();
+        std::cout << at1 << "-" << at2 << "  ";
+        r = m_rotorList.NextRotor(it);
+        if (rotcount%4==0 && r){std::cout << std::endl;}
+        ++rotcount;
+      }
+      std::cout << std::endl;
+      return false;
+    }
+    // Print those that are fixed
+    if (!m_fixedBonds.IsEmpty()){
+      std::cout << "Fixed Rotors: " << std::endl;
+      int end_it = m_fixedBonds.EndBit();
+      int it = m_fixedBonds.FirstBit();
+      int rotcount = 1;
+      while(it!=end_it){
+        OBBond* b = m_mol.GetBond(it);
+        int at1,at2;
+        at1 = b->GetBeginAtomIdx();
+        at2 = b->GetEndAtomIdx();
+        std::cout << at1 << "-" << at2 << "  ";
+        it = m_fixedBonds.NextBit(it);
+        if (rotcount%4==0 && it!=end_it){std::cout << std::endl;}
+        ++rotcount;
+      }
+      std::cout << std::endl;
+    }
+
     nb_rotors = m_rotorList.Size();
     if (!nb_rotors) { // only one conformer
       return false;
@@ -386,12 +443,12 @@ namespace OpenBabel {
     nb_niches = (m_rotorKeys.size()) / 10;
     if (nb_niches < 3)
       nb_niches = 3;
-    sigma_share = nb_rotors / 3;
-    if (sigma_share < 1)
-      sigma_share ++;
-    niche_radius =  nb_rotors / 4;
-    if (niche_radius < 1)
-      niche_radius++;
+    sigma_share = (double)nb_rotors / 3.0;
+    if (sigma_share < 1.0)
+      sigma_share = 1.0;
+    niche_radius =  (double)nb_rotors / 4.0;
+    if (niche_radius < 1.0)
+      niche_radius = 1.0;
 
     return true;
   }
@@ -577,6 +634,10 @@ namespace OpenBabel {
           // make the selection
           score = MakeSelection();
         }
+      if (OB_ISNAN(score)) {
+          (*m_logstream) << "The current score is not a number, will not continue."  << endl  << endl;
+          return;
+      }
       if (i == 0)
         last_score = score;
 
@@ -728,9 +789,9 @@ namespace OpenBabel {
     for (i = 1; i <= m_rotorList.Size(); ++i, rotor = m_rotorList.NextRotor(ri))
       {
         neighbor = best;
-        new_val = unique_generator.NextInt() % rotor->GetResolution().size();
+        new_val = ((OBRandom*)d)->NextInt() % rotor->GetResolution().size();
         while (new_val == best[i])
-          new_val = unique_generator.NextInt() % rotor->GetResolution().size();
+          new_val = ((OBRandom*)d)->NextInt() % rotor->GetResolution().size();
         neighbor[i] = new_val;
         if (IsUniqueKey(backup_population, neighbor) && IsGood(neighbor))
           m_rotorKeys.push_back (neighbor);
@@ -782,30 +843,30 @@ namespace OpenBabel {
       return 0;
 
     // Make a 2-tournament selection to choose first parent
-    i = unique_generator.NextInt() % pop_size;
-    j = unique_generator.NextInt() % pop_size;
+    i = ((OBRandom*)d)->NextInt() % pop_size;
+    j = ((OBRandom*)d)->NextInt() % pop_size;
     parent1 = vshared_fitnes[i] > vshared_fitnes[j] ? i : j;
     iniche = niche_map[parent1];
     if (iniche > -1)
       nsize = dynamic_niches[iniche].size (); // Belongs to a specific niche
 
     // Do we apply crossover here?
-    flag_crossover = (unique_generator.NextFloat () <= p_crossover);
-    if (flag_crossover && (unique_generator.NextFloat () <= niche_mating)  &&  nsize > 1)
+    flag_crossover = (((OBRandom*)d)->NextFloat () <= p_crossover);
+    if (flag_crossover && (((OBRandom*)d)->NextFloat () <= niche_mating)  &&  nsize > 1)
       {
         // Apply niche mating: draw second parent in the same niche, if its has
         // at least 2 members. Make a 2-tournament selection whithin this niche
-        rnd1 =  unique_generator.NextInt() % nsize;
+        rnd1 = ((OBRandom*)d)->NextInt() % nsize;
         i =  dynamic_niches[iniche][rnd1];
-        rnd2 =  unique_generator.NextInt() % nsize;
+        rnd2 = ((OBRandom*)d)->NextInt() % nsize;
         j = dynamic_niches[iniche][rnd2];
         parent2 = vshared_fitnes[i] > vshared_fitnes[j] ? i : j;
       }
     else
       {
         // Draw second in the whole population
-        i = unique_generator.NextInt() % pop_size;
-        j = unique_generator.NextInt() % pop_size;
+        i = ((OBRandom*)d)->NextInt() % pop_size;
+        j = ((OBRandom*)d)->NextInt() % pop_size;
         parent2 = vshared_fitnes[i] > vshared_fitnes[j] ? i : j;
       }
 
@@ -814,7 +875,7 @@ namespace OpenBabel {
         // Cross the 2 vectors: toss a coin for each position (i.e. uniform crossover)
         for (i = 1; i < key1.size(); i++)
           {
-            if ( unique_generator.NextInt() % 2)
+            if (((OBRandom*)d)->NextInt() % 2)
               { // Copy parent1 to offspring 1
                 key1[i] = m_rotorKeys[parent1][i];
                 key2[i] = m_rotorKeys[parent2][i];
@@ -836,10 +897,10 @@ namespace OpenBabel {
     rotor = m_rotorList.BeginRotor(ri);
     for (i = 1; i <= m_rotorList.Size(); ++i, rotor = m_rotorList.NextRotor(ri))
       {
-        if (unique_generator.NextInt() % m_mutability == 0)
-          key1[i] = unique_generator.NextInt() % rotor->GetResolution().size();
-        if (unique_generator.NextInt() % m_mutability == 0)
-          key2[i] = unique_generator.NextInt() % rotor->GetResolution().size();
+        if (((OBRandom*)d)->NextInt() % m_mutability == 0)
+          key1[i] = ((OBRandom*)d)->NextInt() % rotor->GetResolution().size();
+        if (((OBRandom*)d)->NextInt() % m_mutability == 0)
+          key2[i] = ((OBRandom*)d)->NextInt() % rotor->GetResolution().size();
       }
     if (IsUniqueKey(m_rotorKeys, key1) && IsGood(key1))
       ret_code += 1;
@@ -977,7 +1038,8 @@ namespace OpenBabel {
         for (iniche = 0; iniche < dynamic_niches.size (); iniche++)
           {
             j = dynamic_niches[iniche][0];
-            if (key_distance (m_rotorKeys[j], m_rotorKeys[i]) <= niche_radius)
+            dist = key_distance(m_rotorKeys[j], m_rotorKeys[i]);
+            if ((double)dist <= niche_radius)
               {
                 dynamic_niches[iniche].push_back (i);
                 break;
@@ -997,7 +1059,7 @@ namespace OpenBabel {
                 for (j = 0; j < pop_size; j++)
                   {
                     dist = key_distance (m_rotorKeys[i], m_rotorKeys[j]);
-                    if (dist < sigma_share)
+                    if ((double)dist < sigma_share)
                       {
                         sh_ij = 1.0 - pow (((double) dist) / ((double ) sigma_share), alpha_share);
                         sh_count += sh_ij;
