@@ -23,7 +23,12 @@ GNU General Public License for more details.
 #include <openbabel/mol.h>
 #include <openbabel/atom.h>
 #include <openbabel/bond.h>
+#include <openbabel/ring.h>
+#include <openbabel/obiter.h>
 #include <openbabel/builder.h>
+#include <openbabel/elements.h>
+#include <openbabel/generic.h>
+#include "rand.h"
 
 #include <openbabel/stereo/stereo.h>
 #include <openbabel/stereo/cistrans.h>
@@ -474,7 +479,7 @@ namespace OpenBabel {
         float uBounds = _d->GetUpperBounds(path[a] - 1, path[d] - 1) - DIST14_TOL;
 
         bc = _mol.GetBond(path[b], path[c]);
-        if (bc->IsDouble() || bc->IsAromatic()) {
+        if (bc->IsAromatic() || bc->GetBondOrder() == 2) {
           uBounds = lBounds + DIST14_TOL;
           // Correct non-ring neighbors too -- these should be out of the ring
           FOR_NBORS_OF_ATOM(nbr, _mol.GetAtom(path[b])) {
@@ -492,7 +497,7 @@ namespace OpenBabel {
                                _d->GetUpperBounds(nbr->GetIdx() - 1, path[a] - 1) - DIST14_TOL);
           }
 
-        } else if (bc->IsSingle()) {
+        } else if (bc->GetBondOrder() == 1) {
           // Could be anywhere from pure-cis to halfway to trans
           uBounds = _d->GetAvgBounds(path[a] - 1, path[d] - 1);
 
@@ -638,7 +643,7 @@ namespace OpenBabel {
       // For neighbors of d
       //  Actually depends on stereo of bond C-D
       cd = _mol.GetBond(c, d);
-      if (cd && cd->IsDouble()) {
+      if (cd && cd->GetBondOrder() == 2 && !cd->IsAromatic()) {
         stereo = GetCisTransStereo(cd);
       }
       FOR_NBORS_OF_ATOM(e, d) {
@@ -675,7 +680,7 @@ namespace OpenBabel {
       //  Now depends on stereo of bond a-b
       stereo = NULL; // reset
       ab = _mol.GetBond(a, b);
-      if (ab && ab->IsDouble()) {
+      if (ab && ab->GetBondOrder() == 2 && !ab->IsAromatic()) {
         stereo = GetCisTransStereo(ab);
       }
       FOR_NBORS_OF_ATOM(z, a) {
@@ -740,7 +745,7 @@ namespace OpenBabel {
   //! Implements the smoothing described by
   //! Dress, AWM, Havel TF; Discrete Applied Mathematics (1988) v. 19 pp. 129-144
   //! "Shortest Path Problems and Molecular Conformation"
-  //! http://dx.doi.org/10.1016/0166-218X(88)90009-1
+  //! https://doi.org/10.1016/0166-218X(88)90009-1
   void OBDistanceGeometry::TriangleSmooth(int iterations)
   {
     int a, b, c;
@@ -782,11 +787,11 @@ namespace OpenBabel {
           }
 
           // Triangle rule: length can't be shorter than the difference between the legs
-          if (l_bc < (l_ab - u_ac)) {
-            l_bc = l_ab - u_ac;
+          if (l_bc < (l_ab - l_ac)) {
+            l_bc = l_ab - l_ac;
             _d->SetLowerBounds(b, c, l_bc);
-          } else if (l_bc < (l_ac - u_ab)) {
-            l_bc = l_ac - u_ab;
+          } else if (l_bc < (l_ac - l_ab)) {
+            l_bc = l_ac - l_ab;
             _d->SetLowerBounds(b, c, l_bc);
           }
 
@@ -811,12 +816,12 @@ namespace OpenBabel {
 
     for (unsigned int i = 0; i < N; ++i) {
       a = _mol.GetAtom(i+1);
-      aRad = etab.GetVdwRad(a->GetAtomicNum());
+      aRad = OBElements::GetVdwRad(a->GetAtomicNum());
 
       for (unsigned int j = i + 1; j < N; ++j)
         {
           b = _mol.GetAtom(j + 1);
-          bRad = etab.GetVdwRad(b->GetAtomicNum());
+          bRad = OBElements::GetVdwRad(b->GetAtomicNum());
           minDist = aRad + bRad;
           if (minDist < 1.0f)
             minDist = 1.0f;
@@ -1189,7 +1194,7 @@ namespace OpenBabel {
 
     for (unsigned int i = 1; i <= _mol.NumAtoms(); ++i) {
       a = _mol.GetAtom(i);
-      aRad = etab.GetVdwRad(a->GetAtomicNum());
+      aRad = OBElements::GetVdwRad(a->GetAtomicNum());
       for (unsigned int j = i + 1; j <= _mol.NumAtoms(); ++j) {
           b = _mol.GetAtom(j);
 
@@ -1207,7 +1212,7 @@ namespace OpenBabel {
           if (_mol.GetBond(a, b))
             continue;
 
-          bRad = etab.GetVdwRad(b->GetAtomicNum());
+          bRad = OBElements::GetVdwRad(b->GetAtomicNum());
           minDist = aRad + bRad - 2.5;
           if (minDist < 0.8)
             minDist = 0.8;
