@@ -15,11 +15,19 @@ This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
-***********************************************************************/
+************************************************************************/
 
 #include <openbabel/babelconfig.h>
 #include <openbabel/mol.h>
 #include <openbabel/locale.h>
+#include <openbabel/elements.h>
+#include <openbabel/atom.h>
+#include <openbabel/obiter.h>
+#include <openbabel/generic.h>
+#include <openbabel/bond.h>
+#include <openbabel/parsmart.h>
+
+#include <cstdlib>
 
 #include "forcefielduff.h"
 
@@ -694,22 +702,22 @@ namespace OpenBabel {
       // calculate the number of lone pairs
       // e.g. for IF3 => "T-shaped"
       valenceElectrons -= b->GetFormalCharge(); // make sure to look for I+F4 -> see-saw
-      double lonePairs = (valenceElectrons - b->BOSum()) / 2.0;
+      double lonePairs = (valenceElectrons - b->GetExplicitValence()) / 2.0;
       // we actually need to round up here -- single e- take room too.
       int sites = (int)ceil(lonePairs);
-      coordination = b->GetValence() + sites;
+      coordination = b->GetExplicitDegree() + sites;
       if (coordination <= 4) { // normal valency
         coordination = ipar;
-      } else if (b->IsSulfur() && b->CountFreeOxygens() == 3) {
+      } else if (b->GetAtomicNum() == OBElements::Sulfur && b->CountFreeOxygens() == 3) {
         // SO3, should be planar
         // PR#2971473, thanks to Philipp Rumpf
         coordination = 2; // i.e., sp2
       }
       /* planar coordination of hexavalent molecules.*/
-      if (lonePairs == 0 && b->GetValence() == 3 && b->BOSum() == 6) {
+      if (lonePairs == 0 && b->GetExplicitDegree() == 3 && b->GetExplicitValence() == 6) {
         coordination = 2;
       }
-      if (lonePairs == 0 && b->GetValence() == 7) {
+      if (lonePairs == 0 && b->GetExplicitDegree() == 7) {
         coordination = 7;
       }
       // Check to see if coordination is really correct
@@ -718,13 +726,13 @@ namespace OpenBabel {
     } else {
       coordination = ipar; // coordination of central atom
     }
-    if (b->GetValence() > 4) {
-      coordination = b->GetValence();
+    if (b->GetExplicitDegree() > 4) {
+      coordination = b->GetExplicitDegree();
     } else {
-      int coordDifference = ipar - b->GetValence();
+      int coordDifference = ipar - b->GetExplicitDegree();
       if (abs(coordDifference) > 2)
         // low valent, but very different than expected by ipar
-        coordination = b->GetValence() - 1; // 4 coordinate == sp3
+        coordination = b->GetExplicitDegree() - 1; // 4 coordinate == sp3
     }
     return coordination;
   }
@@ -864,7 +872,7 @@ namespace OpenBabel {
       if (HasGroups()) {
         bool validBond = false;
         for (unsigned int i=0; i < _intraGroup.size(); ++i) {
-          if (_intraGroup[i].BitIsOn(a->GetIdx()) && _intraGroup[i].BitIsOn(b->GetIdx()))
+          if (_intraGroup[i].BitIsSet(a->GetIdx()) && _intraGroup[i].BitIsSet(b->GetIdx()))
             validBond = true;
         }
         if (!validBond)
@@ -935,8 +943,8 @@ namespace OpenBabel {
       if (HasGroups()) {
         bool validAngle = false;
         for (unsigned int i=0; i < _intraGroup.size(); ++i) {
-          if (_intraGroup[i].BitIsOn(a->GetIdx()) && _intraGroup[i].BitIsOn(b->GetIdx()) &&
-              _intraGroup[i].BitIsOn(c->GetIdx()))
+          if (_intraGroup[i].BitIsSet(a->GetIdx()) && _intraGroup[i].BitIsSet(b->GetIdx()) &&
+              _intraGroup[i].BitIsSet(c->GetIdx()))
             validAngle = true;
         }
         if (!validAngle)
@@ -1106,7 +1114,7 @@ namespace OpenBabel {
 
 			// Equation 13 from paper -- corrected by Towhee
 			// Note that 1/(rij * rjk) cancels with rij*rjk in eqn. 13
-			anglecalc.ka = (644.12 * KCAL_TO_KJ) * (anglecalc.zi * anglecalc.zk / (pow(rac, 5.0)));
+			anglecalc.ka = (664.12 * KCAL_TO_KJ) * (anglecalc.zi * anglecalc.zk / (pow(rac, 5.0)));
 			anglecalc.ka *= (3.0*rab*rbc*(1.0 - anglecalc.cosT0*anglecalc.cosT0) - rac*rac*anglecalc.cosT0);
       // Make sure to divide by n^2 to save CPU cycles
       switch (anglecalc.coord) {
@@ -1152,8 +1160,8 @@ namespace OpenBabel {
       if (HasGroups()) {
         bool validTorsion = false;
         for (unsigned int i=0; i < _intraGroup.size(); ++i) {
-          if (_intraGroup[i].BitIsOn(a->GetIdx()) && _intraGroup[i].BitIsOn(b->GetIdx()) &&
-              _intraGroup[i].BitIsOn(c->GetIdx()) && _intraGroup[i].BitIsOn(d->GetIdx()))
+          if (_intraGroup[i].BitIsSet(a->GetIdx()) && _intraGroup[i].BitIsSet(b->GetIdx()) &&
+              _intraGroup[i].BitIsSet(c->GetIdx()) && _intraGroup[i].BitIsSet(d->GetIdx()))
             validTorsion = true;
         }
         if (!validTorsion)
@@ -1298,7 +1306,7 @@ namespace OpenBabel {
         continue;
       }
 
-      if (b->GetValence() > 3) // no OOP for hypervalent atoms
+      if (b->GetExplicitDegree() > 3) // no OOP for hypervalent atoms
         continue;
 
       a = NULL;
@@ -1361,10 +1369,10 @@ namespace OpenBabel {
       if (HasGroups()) {
         bool validOOP = false;
         for (unsigned int i=0; i < _intraGroup.size(); ++i) {
-          if (_intraGroup[i].BitIsOn(a->GetIdx()) &&
-              _intraGroup[i].BitIsOn(b->GetIdx()) &&
-              _intraGroup[i].BitIsOn(c->GetIdx()) &&
-              _intraGroup[i].BitIsOn(d->GetIdx()))
+          if (_intraGroup[i].BitIsSet(a->GetIdx()) &&
+              _intraGroup[i].BitIsSet(b->GetIdx()) &&
+              _intraGroup[i].BitIsSet(c->GetIdx()) &&
+              _intraGroup[i].BitIsSet(d->GetIdx()))
             validOOP = true;
         }
         if (!validOOP)
@@ -1429,13 +1437,13 @@ namespace OpenBabel {
       if (HasGroups()) {
         bool validVDW = false;
         for (unsigned int i=0; i < _interGroup.size(); ++i) {
-          if (_interGroup[i].BitIsOn(a->GetIdx()) && _interGroup[i].BitIsOn(b->GetIdx()))
+          if (_interGroup[i].BitIsSet(a->GetIdx()) && _interGroup[i].BitIsSet(b->GetIdx()))
             validVDW = true;
         }
         for (unsigned int i=0; i < _interGroups.size(); ++i) {
-          if (_interGroups[i].first.BitIsOn(a->GetIdx()) && _interGroups[i].second.BitIsOn(b->GetIdx()))
+          if (_interGroups[i].first.BitIsSet(a->GetIdx()) && _interGroups[i].second.BitIsSet(b->GetIdx()))
             validVDW = true;
-          if (_interGroups[i].first.BitIsOn(b->GetIdx()) && _interGroups[i].second.BitIsOn(a->GetIdx()))
+          if (_interGroups[i].first.BitIsSet(b->GetIdx()) && _interGroups[i].second.BitIsSet(a->GetIdx()))
             validVDW = true;
         }
 
@@ -1493,13 +1501,13 @@ namespace OpenBabel {
       if (HasGroups()) {
         bool validEle = false;
         for (unsigned int i=0; i < _interGroup.size(); ++i) {
-          if (_interGroup[i].BitIsOn(a->GetIdx()) && _interGroup[i].BitIsOn(b->GetIdx()))
+          if (_interGroup[i].BitIsSet(a->GetIdx()) && _interGroup[i].BitIsSet(b->GetIdx()))
             validEle = true;
         }
         for (unsigned int i=0; i < _interGroups.size(); ++i) {
-          if (_interGroups[i].first.BitIsOn(a->GetIdx()) && _interGroups[i].second.BitIsOn(b->GetIdx()))
+          if (_interGroups[i].first.BitIsSet(a->GetIdx()) && _interGroups[i].second.BitIsSet(b->GetIdx()))
             validEle = true;
-          if (_interGroups[i].first.BitIsOn(b->GetIdx()) && _interGroups[i].second.BitIsOn(a->GetIdx()))
+          if (_interGroups[i].first.BitIsSet(b->GetIdx()) && _interGroups[i].second.BitIsSet(a->GetIdx()))
             validEle = true;
         }
 
