@@ -22,13 +22,19 @@ GNU General Public License for more details.
 #include <iterator>
 #include <locale>
 
-#include "openbabel/mol.h"
+#include <openbabel/mol.h>
+#include <openbabel/atom.h>
+#include <openbabel/elements.h>
+#include <openbabel/obiter.h>
+
 #include "openbabel/oberror.h"
 #include "openbabel/obconversion.h"
 #include "openbabel/reaction.h"
 #include "openbabel/kinetics.h"
 #include "openbabel/obmolecformat.h"
 
+#include <cstdlib>
+
 using namespace std;
 
 namespace OpenBabel
@@ -819,7 +825,7 @@ bool ChemKinFormat::WriteHeader(OBConversion* pConv)
     if(strcmp(title, "M"))
       species.push_back(title);
     FOR_ATOMS_OF_MOL(atom, itr->get())
-      elements.insert(etab.GetSymbol(atom->GetAtomicNum()));
+      elements.insert(OBElements::GetSymbol(atom->GetAtomicNum()));
   }
   if(!elements.empty())
   {
