@@ -16,8 +16,20 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/
 
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
 #include <openbabel/babelconfig.h>
 #include <openbabel/obmolecformat.h>
+#include <openbabel/mol.h>
+#include <openbabel/atom.h>
+#include <openbabel/elements.h>
+#include <openbabel/bond.h>
+#include <openbabel/obiter.h>
+#include <openbabel/generic.h>
+#include <cstdlib>
+
 #include <openbabel/math/spacegroup.h>
 
 #include <sstream>
@@ -1296,13 +1308,13 @@ namespace OpenBabel
   // Returns: true if the atom is an oxygen and connected to two hydrogens and up to one other atom
   bool CIFisWaterOxygen(OBAtom *atom)
   {
-    if (!atom->IsOxygen())
+    if (atom->GetAtomicNum() != OBElements::Oxygen)
       return false;
 
     int nonHydrogenCount = 0;
     int hydrogenCount = 0;
     FOR_NBORS_OF_ATOM(neighbor, *atom) {
-      if (!neighbor->IsHydrogen())
+      if (neighbor->GetAtomicNum() != OBElements::Hydrogen)
         nonHydrogenCount++;
       else
         hydrogenCount++;
@@ -1322,7 +1334,7 @@ namespace OpenBabel
     FOR_ATOMS_OF_MOL(atom, *mol) {
 
       if ((atom->GetAtomicNum() == 7 || atom->GetAtomicNum() == 15)
-          && atom->BOSum() == 4) {
+          && atom->GetExplicitValence() == 4) {
         // check if we should make a positive charge?
         // i.e., 4 non-metal neighbors
         bool nonMetalNeighbors = true;
@@ -1349,7 +1361,7 @@ namespace OpenBabel
         continue;
 
       // If we're connected to anything besides H2O, keep going
-      if (atom->GetValence() != 0) {
+      if (atom->GetExplicitDegree() != 0) {
         int nonWaterBonds = 0;
         FOR_NBORS_OF_ATOM(neighbor, &*atom) {
           if (!CIFisWaterOxygen(&*neighbor)) {
@@ -1468,7 +1480,7 @@ namespace OpenBabel
               if(nbc>0) tmpSymbol=tmpSymbol.substr(0,nbc);
               else tmpSymbol="C";//Something went wrong, no symbol ! Default to C ??
 
-              int atomicNum = etab.GetAtomicNum(tmpSymbol.c_str());
+              int atomicNum = OBElements::GetAtomicNum(tmpSymbol.c_str());
               // Test for some oxygens with subscripts
               if (atomicNum == 0 && tmpSymbol[0] == 'O') {
                 atomicNum = 8; // e.g. Ob, OH, etc.
@@ -1626,12 +1638,12 @@ namespace OpenBabel
            }
          else
            {
-             label_str = etab.GetSymbol(atom->GetAtomicNum()) + to_string(i);
+             label_str = OBElements::GetSymbol(atom->GetAtomicNum()) + to_string(i);
              i++;
            }
 
          snprintf(buffer, BUFF_SIZE, "    %-8s%-5s%.5f%10.5f%10.5f%8.3f\n",
-                  label_str.c_str(), etab.GetSymbol(atom->GetAtomicNum()),
+                  label_str.c_str(), OBElements::GetSymbol(atom->GetAtomicNum()),
                   X, Y, Z, occup);
 
          ofs << buffer;
