@@ -16,6 +16,14 @@ GNU General Public License for more details.
 #include <openbabel/babelconfig.h>
 
 #include <openbabel/obmolecformat.h>
+#include <openbabel/mol.h>
+#include <openbabel/atom.h>
+#include <openbabel/bond.h>
+#include <openbabel/obiter.h>
+#include <openbabel/elements.h>
+#include <openbabel/generic.h>
+#include <cstdlib>
+
 #ifdef _MSC_VER
 #include <regex>
 #else
@@ -80,7 +88,9 @@ namespace OpenBabel
     {
       return
         "ORCA input format\n"
-        "This can be used as a template file for orca calculations\n";
+        "Write Options e.g. -xk\n"
+        "  k  \"keywords\" Use the specified keywords for input\n"
+        "  f    <file>     Read the file specified for input keywords\n\n";
     }
 
     virtual const char* SpecificationURL()
@@ -198,7 +208,7 @@ namespace OpenBabel
 
                 if (newMol){
                     atom = mol.NewAtom();
-                    atom->SetAtomicNum(etab.GetAtomicNum(vs[0].c_str()));                //set atomic number
+                    atom->SetAtomicNum(OBElements::GetAtomicNum(vs[0].c_str()));                //set atomic number
                     atom->SetVector(x,y,z); //set atom coordinates
                 }
                 if (geoOptRun){
@@ -517,7 +527,7 @@ namespace OpenBabel
                     atom->SetVector(x,y,z); //set coordinates
 
                     //set atomic number
-                    atom->SetAtomicNum(etab.GetAtomicNum(vs[0].c_str()));
+                    atom->SetAtomicNum(OBElements::GetAtomicNum(vs[0].c_str()));
 
                     if (!ifs.getline(buffer,BUFF_SIZE))
                         break;
@@ -643,18 +653,37 @@ namespace OpenBabel
     ostream &ofs = *pConv->GetOutStream();
     OBMol &mol = *pmol;
 
-    char buffer[BUFF_SIZE];
-
     ofs << "# ORCA input file" << endl;
     ofs << "# " << mol.GetTitle() << endl;
-    ofs << "! insert inline commands here " << endl;
+
+    const char *keywords = pConv->IsOption("k",OBConversion::OUTOPTIONS);
+    const char *keywordFile = pConv->IsOption("f",OBConversion::OUTOPTIONS);
+    string defaultKeywords = "! insert inline commands here ";
+
+    if(keywords)
+      {
+        defaultKeywords = keywords;
+      }
+    if (keywordFile)
+      {
+        ifstream kfstream(keywordFile);
+        string keyBuffer;
+        if (kfstream)
+          {
+            while (getline(kfstream, keyBuffer))
+              ofs << keyBuffer << endl;
+          }
+      }
+    else
+      ofs << defaultKeywords << endl;
+
     ofs << "* xyz " << mol.GetTotalCharge() << " " << mol.GetTotalSpinMultiplicity() << endl;
 
 
     FOR_ATOMS_OF_MOL(atom, mol)
     {
         ofs << setw(4) << right
-            << OpenBabel::etab.GetSymbol(atom->GetAtomicNum())
+            << OpenBabel::OBElements::GetSymbol(atom->GetAtomicNum())
             << setw(15) << setprecision(5) << fixed << showpoint
             << right << atom->GetX() << " " << setw(15) << atom->GetY() << " "
             << setw(15) << atom->GetZ() << endl;
