@@ -14,11 +14,20 @@ GNU General Public License for more details.
 
 #include <openbabel/babelconfig.h>
 #include <openbabel/obmolecformat.h>
+#include <openbabel/mol.h>
+#include <openbabel/atom.h>
+#include <openbabel/bond.h>
+#include <openbabel/obiter.h>
+#include <openbabel/elements.h>
+#include <openbabel/generic.h>
+
 
 #include <limits>
 #include <locale> // For isalpha(int)
 #include <functional>
 #include <iostream>
+#include <algorithm>
+#include <cstdlib>
 
 #define EV_TO_KCAL_PER_MOL 23.060538
 
@@ -81,16 +90,20 @@ namespace OpenBabel {
 
         "Both VASP 4.x and 5.x POSCAR formats are supported.\n\n"
 
+	"By default, atoms are written out in the order they are present in the input\n"
+	"molecule. To sort by atomic number specify ``-xw``. To specify the sort\n"
+	"order, use the ``-xz`` option.\n\n"
+
         "Read Options e.g. -as\n"
         "  s Output single bonds only\n"
         "  b Disable bonding entirely\n\n"
 
-        "Write Options e.g. -xw\n"
-        "  Atoms soring:\n"
-        "    no option: default order (presumably this is the order of atoms in the input molecule)\n"
-        "    -xw : Sort atoms by atomic number\n"
-        "    -xz 'atom1 atom2 ..': atom1 first, atom2 second ..., then default order\n"
-        "    -xw -xz 'atom1 atom2': atom1 first, atom2 second ..., then sort atoms by atomic number\n"
+        "Write Options e.g. -x4\n"
+        " w  Sort atoms by atomic number\n"
+        " z <list of atoms>  Specify the order to write out atoms\n"
+	"       'atom1 atom2 ...': atom1 first, atom2 second, etc. The remaining\n"
+	"       atoms are written in the default order or (if ``-xw`` is specified)\n"
+	"       in order of atomic number.\n"
         "  4 Write a POSCAR using the VASP 4.x specification.\n"
         "    The default is to use the VASP 5.x specification.\n\n"
         ;
@@ -250,7 +263,7 @@ namespace OpenBabel {
     if (symbolsInGeometryFile) {
       atomTypes.clear();
       for (size_t i = 0; i < vs.size(); ++i) {
-        atomTypes.push_back(OpenBabel::etab.GetAtomicNum(vs.at(i).c_str()));
+        atomTypes.push_back(OpenBabel::OBElements::GetAtomicNum(vs.at(i).c_str()));
       }
       // Fetch next line to get stoichiometry
       ifs_cont.getline(buffer,BUFF_SIZE);
@@ -270,7 +283,7 @@ namespace OpenBabel {
               str.erase(i,1);
               --i;
             }
-          atomTypes.push_back(OpenBabel::etab.GetAtomicNum(str.c_str()));
+          atomTypes.push_back(OpenBabel::OBElements::GetAtomicNum(str.c_str()));
         }
       }
       ifs_pot.close();
@@ -617,8 +630,6 @@ namespace OpenBabel {
     OBUnitCell *uc = NULL;
     vector<vector3> cell;
 
-    bool selective;
-
     const char * sortAtomsNum = pConv->IsOption("w", OBConversion::OUTOPTIONS);
     const char * sortAtomsCustom = pConv->IsOption("z", OBConversion::OUTOPTIONS);
 
@@ -638,7 +649,7 @@ namespace OpenBabel {
       vector<string> vs;
       tokenize(vs, sortAtomsCustom);
       for(size_t i = 0; i < vs.size(); ++i)
-        custom_sort_nums.push_back(etab.GetAtomicNum(vs[i].c_str()));
+        custom_sort_nums.push_back(OBElements::GetAtomicNum(vs[i].c_str()));
     }
 
     compare_sort_items csi(custom_sort_nums, sortAtomsNum != NULL);
@@ -665,7 +676,7 @@ namespace OpenBabel {
       }
       else
       {    
-        if(atomicNums.size() > 0);  
+        if(atomicNums.size() > 0)  
           atomicNums.rbegin()->second++;
       }  
       
@@ -706,7 +717,7 @@ namespace OpenBabel {
       for (vector< std::pair<int, int> >::const_iterator
            it = atomicNums.begin(),
            it_end = atomicNums.end(); it != it_end; ++it) {
-        snprintf(buffer, BUFF_SIZE, "%-3s ", etab.GetSymbol(it->first));
+        snprintf(buffer, BUFF_SIZE, "%-3s ", OBElements::GetSymbol(it->first));
         ofs << buffer ;
       }
       ofs << endl;
@@ -722,12 +733,11 @@ namespace OpenBabel {
     ofs << endl;
 
     // assume that there are no constraints on the atoms
-    selective = false;
-    OBAtom *atom;
+    bool selective = false;
     // and test if any of the atoms has constraints
     FOR_ATOMS_OF_MOL(atom, mol) {
       if (atom->HasData("move")){
-        selective =true;
+        selective = true;
         break;
       }
     }
@@ -749,8 +759,8 @@ namespace OpenBabel {
       // if at least one atom has info about constraints
       if (selective) {
         // if this guy has, write it out
-        if (atom->HasData("move")) {
-          OBGenericData *cp = (*it)->GetData("move");
+        if ((*it)->HasData("move")) {
+          OBPairData *cp = (OBPairData*)(*it)->GetData("move");
           // seemingly ridiculous number of digits is written out
           // but sometimes you just don't want to change them
           ofs << " " << cp->GetValue().c_str();
