@@ -32,6 +32,12 @@ typedef short int16;
 typedef unsigned short uint16;
 
 #include <openbabel/obmolecformat.h>
+#include <openbabel/mol.h>
+#include <openbabel/atom.h>
+#include <openbabel/elements.h>
+#include <openbabel/bond.h>
+#include <cstdlib>
+
 
 #define mem_alloc malloc
 #define mem_free free
@@ -372,6 +378,7 @@ bool YOBFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
   struct atomid id;
   OBAtom *dstatom;
   OBResidue *res;
+  bool has_residue_info = false;
 
   /* VERIFY FILE FORMAT */
   ifs.read(buffer,8);
@@ -404,6 +411,7 @@ bool YOBFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
     dstatom->SetVector(pos);
     if (!mob_hasres(srcatom,&id))
     { /* NEW RESIDUE FOUND */
+      has_residue_info = true;
       resstart=srcatom;
       reslen=mob_reslen(resstart,atoms-i);
       mob_getid(&id,srcatom);
@@ -453,6 +461,8 @@ bool YOBFormat::ReadMolecule(OBBase* pOb, OBConversion* pConv)
 
   mol.EndModify();
   if (charged) mol.SetPartialChargesPerceived();
+  if (has_residue_info)
+    mol.SetChainsPerceived();
   if (!mol.NumAtoms()) return(false);
   return(true); }
 
@@ -472,7 +482,7 @@ bool YOBFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
 
   //  bool hetatom;
   char buffer[32],/*resname[4],*/atomname[5];
-  char double1[8]={0,0,0,0,0,0,-16,0x3f};
+  unsigned char double1[8]={0,0,0,0,0,0,0xf0,0x3f};
   //   char *str;
   int i,j,/*m,q,*/pos;
   int /*resno,chainNum,link,linktype,*/atoms,element,links/*,chain*/;
@@ -500,7 +510,7 @@ bool YOBFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
   mem_set(buffer,0,8);
   for (i=0;i<4;i++)
   { for (j=0;j<4;j++)
-    { if (i==j) ofs.write(double1,8);
+    { if (i==j) ofs.write((char*)double1,8);
       else ofs.write(buffer,8); } }
   storeint32le(buffer,MOB_INFOEND);
   storeint32le(&buffer[4],MOB_INFOENDSIZE);
@@ -539,7 +549,7 @@ bool YOBFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
     //printf("Babel atom %d with %d links:\n",i,links);
     for (linkedatom=srcatom->BeginNbrAtom(iter);linkedatom;linkedatom=srcatom->NextNbrAtom(iter))
     { storeint32le(buffer,linkedatom->GetIdx()-1);
-      bondorder=(*iter)->GetBO();
+      bondorder=(*iter)->GetBondOrder();
       //printf("  Order %d\n",bondorder);
       if (bondorder==4) bondorder=MOB_LINKQUADRUPLE;
       else if (bondorder==5) bondorder=MOB_LINKRESONANCE50;
@@ -561,7 +571,7 @@ bool YOBFormat::WriteMolecule(OBBase* pOb, OBConversion* pConv)
       str_copy(&buffer[8],(char*)res->GetName().c_str());
       snprintf(&buffer[12], 4, "%4d",res->GetNum()); }
     else
-    { str_copy(&buffer[4],etab.GetSymbol(element));
+    { str_copy(&buffer[4],OBElements::GetSymbol(element));
       str_copy(&buffer[8],"UNK    1"); }
     for (j=4;j<16;j++) if (!buffer[j]) buffer[j]=' ';
     ofs.write(buffer,16); }
