@@ -23,11 +23,15 @@ GNU General Public License for more details.
 #include <set>
 
 #include <openbabel/mol.h>
+#include <openbabel/atom.h>
+#include <openbabel/bond.h>
+#include <openbabel/ring.h>
+#include <openbabel/obiter.h>
 #include <openbabel/generic.h>
 #include <openbabel/math/matrix3x3.h>
+#include <openbabel/elements.h>
 
 // needed for msvc to have at least one reference to AtomClass, AliasData in openbabel library
-#include <openbabel/atomclass.h>
 #include <openbabel/alias.h>
 
 using namespace std;
@@ -239,7 +243,7 @@ namespace OpenBabel
     _bgn(0), _end(0), _ord(0), _stereo(0)
   {  }
 
-  OBVirtualBond::OBVirtualBond(int bgn,int end,int ord,int stereo):
+  OBVirtualBond::OBVirtualBond(unsigned int bgn, unsigned int end, unsigned int ord, int stereo):
     OBGenericData("VirtualBondData", OBGenericDataType::VirtualBondData, perceived),
     _bgn(bgn), _end(end), _ord(ord), _stereo(stereo)
   {  }
@@ -1191,9 +1195,9 @@ namespace OpenBabel
     vector<triple<OBAtom*,OBAtom*,double> >::iterator ad;
     for(ad = _ads.begin();ad != _ads.end() && (Aprotor || Dprotor);++ad)
       {
-        if(!ad->first->IsHydrogen())
+        if (ad->first->GetAtomicNum() != OBElements::Hydrogen)
           Aprotor = false;
-        if(!ad->second->IsHydrogen())
+        if (ad->second->GetAtomicNum() != OBElements::Hydrogen)
           Dprotor = false;
       }
     return (Aprotor || Dprotor);
@@ -1315,124 +1319,6 @@ namespace OpenBabel
     return(true);
   }
 
-  //
-  // Member functions for OBChiralDarta
-  //
-  bool OBChiralData::SetAtom4Refs(std::vector<unsigned int> atom4refs, atomreftype t)
-  {
-    if (atom4refs.size() != 4)
-      {
-        obErrorLog.ThrowError(__FUNCTION__, "Incorrect number of atoms atom4refs, should be 4", obDebug);
-        return(false);
-      }
-    switch(t){
-    case input: _atom4refs = atom4refs;break;
-    case output:_atom4refo = atom4refs;break;
-    case calcvolume:_atom4refc = atom4refs;break;
-    default:
-      obErrorLog.ThrowError(__FUNCTION__, "AtomRefType called is invalid", obDebug);
-      return(false);
-    }
-    return (true);
-  }
-
-  int OBChiralData::AddAtomRef(unsigned int atomref, atomreftype t)
-  {
-    switch(t){
-    case input: _atom4refs.push_back(atomref);break;
-    case output: _atom4refo.push_back(atomref);break;
-    case calcvolume:_atom4refc.push_back(atomref);break;
-    default:
-      obErrorLog.ThrowError(__FUNCTION__, "AtomRefType called is invalid", obDebug);
-      return(false);
-    }
-
-    return (_atom4refs.size());
-  }
-
-  unsigned int OBChiralData::GetAtomRef(int a, atomreftype t)
-  {
-    switch(t){
-    case input: return(_atom4refs[a]);break;
-    case output: return(_atom4refo[a]);break;
-    case calcvolume: return(_atom4refc[a]);break;
-    default:
-      obErrorLog.ThrowError(__FUNCTION__, "AtomRefType called is invalid", obDebug);
-      return(false);
-    }
-  }
-  std::vector<unsigned int> OBChiralData::GetAtom4Refs(atomreftype t) const
-  {
-    switch (t){
-    case output:
-      return(_atom4refo);
-      break;
-    case input:
-      return(_atom4refs);
-      break;
-    case calcvolume:
-      return(_atom4refc);
-      break;
-    default:
-      obErrorLog.ThrowError(__FUNCTION__, "AtomRefType called is invalid", obDebug);
-      return(_atom4refo);
-    }
-  }
-
-  unsigned int OBChiralData::GetSize(atomreftype t) const
-  {
-    switch (t)
-      {
-      case output:
-        return(unsigned int)_atom4refo.size();
-        break;
-      case input:
-        return(unsigned int)_atom4refs.size();
-        break;
-      case calcvolume:
-        return(unsigned int)_atom4refc.size();
-      default:
-        obErrorLog.ThrowError(__FUNCTION__, "AtomRefType called is invalid", obDebug);
-        return(0);
-      }
-  }
-
-  // Chiral data is a perceived data type. We might read in some chiral info
-  // but this class derives and converts from whatever is read
-  OBChiralData::OBChiralData()
-    : OBGenericData("ChiralData", OBGenericDataType::ChiralData, perceived)
-  {  }
-
-  OBChiralData::OBChiralData(const OBChiralData &src)
-    : OBGenericData(src)
-  {
-    _atom4refs = src._atom4refs;
-    _atom4refo = src._atom4refo;
-    _atom4refc = src._atom4refc;
-    parity     = src.parity;
-  }
-
-  OBChiralData & OBChiralData::operator=(const OBChiralData &src)
-  {
-    if(this == &src)
-      return(*this);
-
-    _source = src._source;
-
-    _atom4refs = src._atom4refs;
-    _atom4refo = src._atom4refo;
-    _atom4refc = src._atom4refc;
-    parity=src.parity;
-    return(*this);
-  }
-
-  void OBChiralData::Clear()
-  {
-    _atom4refs.clear();
-    parity=0;
-    _atom4refo.clear();
-    _atom4refc.clear();
-  }
 
 //
 //member functions for OBDOSData class
@@ -1458,14 +1344,12 @@ void OBDOSData::SetData(double fermi,
 
   // member functions for OBOrbitalData
 
-  void OBOrbitalData::LoadClosedShellOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, int alphaHOMO)
+  void OBOrbitalData::LoadClosedShellOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, unsigned int alphaHOMO)
   {
     if (energies.size() < symmetries.size())
       return; // something is very weird -- it's OK to pass no symmetries (we'll assume "A")
     if (energies.size() == 0)
       return;
-    if (alphaHOMO <= 0)
-      return;
     if (alphaHOMO > energies.size())
       return;
 
@@ -1491,14 +1375,12 @@ void OBDOSData::SetData(double fermi,
       }
   }
 
-  void OBOrbitalData::LoadAlphaOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, int alphaHOMO)
+  void OBOrbitalData::LoadAlphaOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, unsigned int alphaHOMO)
   {
     if (energies.size() < symmetries.size())
       return; // something is very weird -- it's OK to pass no symmetries (we'll assume "A")
     if (energies.size() == 0)
       return;
-    if (alphaHOMO <= -1)
-      return;
     if (alphaHOMO > energies.size())
       return;
 
@@ -1522,14 +1404,12 @@ void OBDOSData::SetData(double fermi,
       }
   }
 
-  void OBOrbitalData::LoadBetaOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, int betaHOMO)
+  void OBOrbitalData::LoadBetaOrbitals(std::vector<double> energies, std::vector<std::string> symmetries, unsigned int betaHOMO)
   {
     if (energies.size() < symmetries.size())
       return; // something is very weird -- it's OK to pass no symmetries (we'll assume "A")
     if (energies.size() == 0)
       return;
-    if (betaHOMO <= -1)
-      return;
     if (betaHOMO > energies.size())
       return;
 
@@ -1642,6 +1522,11 @@ unsigned int OBVibrationData::GetNumberOfFrequencies() const
   return this->_vFrequencies.size();
 }
 
+void OBFreeGrid::Clear()
+{
+  _points.clear();
+}
+
 } //end namespace OpenBabel
 
 //! \file generic.cpp
