@@ -20,8 +20,11 @@ GNU General Public License for more details.
 
 #include <ctype.h>
 #include <iomanip>
+#include <cstring>
 
 #include <openbabel/mol.h>
+#include <openbabel/atom.h>
+#include <openbabel/bond.h>
 #include <openbabel/parsmart.h>
 #include <openbabel/stereo/stereo.h>
 #include <openbabel/stereo/tetrahedral.h>
@@ -30,7 +33,6 @@ using namespace std;
 
 namespace OpenBabel
 {
-
   /*! \class OBSmartsPattern parsmart.h <openbabel/parsmart.h>
 
     Substructure search is an incredibly useful tool in the context of a
@@ -148,7 +150,6 @@ namespace OpenBabel
 #define BE_DOWNUNSPEC   16
 
 
-  static int GetVectorBinding();
   static int CreateAtom(Pattern*,AtomExpr*,int,int vb=0);
 
   const int SmartsImplicitRef = -9999; // Used as a placeholder when recording atom nbrs for chiral atoms
@@ -174,7 +175,7 @@ namespace OpenBabel
 
   static AtomExpr *CopyAtomExpr( AtomExpr *expr )
   {
-    register AtomExpr *result;
+    AtomExpr *result;
 
     result = new AtomExpr;
     result->type = expr->type;
@@ -230,7 +231,7 @@ namespace OpenBabel
 
   static AtomExpr *BuildAtomPred( int type )
   {
-    register AtomExpr *result;
+    AtomExpr *result;
 
     result = new AtomExpr;
     result->leaf.type = type;
@@ -240,7 +241,7 @@ namespace OpenBabel
 
   static AtomExpr *BuildAtomLeaf( int type, int val )
   {
-    register AtomExpr *result;
+    AtomExpr *result;
 
     result = new AtomExpr;
     result->leaf.type = type;
@@ -250,7 +251,7 @@ namespace OpenBabel
 
   static AtomExpr *BuildAtomNot( AtomExpr *expr )
   {
-    register AtomExpr *result;
+    AtomExpr *result;
 
     result = new AtomExpr;
     result->mon.type = AE_NOT;
@@ -260,7 +261,7 @@ namespace OpenBabel
 
   static AtomExpr *BuildAtomBin( int op, AtomExpr *lft, AtomExpr *rgt )
   {
-    register AtomExpr *result;
+    AtomExpr *result;
 
     result = new AtomExpr;
     result->bin.type = op;
@@ -271,7 +272,7 @@ namespace OpenBabel
 
   static AtomExpr *BuildAtomRecurs( Pattern *pat )
   {
-    register AtomExpr *result;
+    AtomExpr *result;
 
     result = new AtomExpr;
     result->recur.type = AE_RECUR;
@@ -290,18 +291,13 @@ namespace OpenBabel
                 : BuildAtomLeaf(AE_ALIPHELEM,elem);
   }
 
-  static int IsInvalidAtom( AtomExpr *expr )
-  {
-    return !expr || expr->type==AE_FALSE;
-  }
-
   /*================================*/
   /*  Bond Expression Manipulation  */
   /*================================*/
 
   static BondExpr *CopyBondExpr( BondExpr *expr )
   {
-    register BondExpr *result;
+    BondExpr *result;
 
     result = new BondExpr;
     result->type = expr->type;
@@ -384,7 +380,7 @@ namespace OpenBabel
 
   static BondExpr *BuildBondLeaf( int type )
   {
-    register BondExpr *result;
+    BondExpr *result;
 
     result = new BondExpr;
     result->type = type;
@@ -393,7 +389,7 @@ namespace OpenBabel
 
   static BondExpr *BuildBondNot( BondExpr *expr )
   {
-    register BondExpr *result;
+    BondExpr *result;
 
     result = new BondExpr;
     result->mon.type = BE_NOT;
@@ -403,7 +399,7 @@ namespace OpenBabel
 
   static BondExpr *BuildBondBin( int op, BondExpr *lft, BondExpr *rgt )
   {
-    register BondExpr *result;
+    BondExpr *result;
 
     result = new BondExpr;
     result->bin.type = op;
@@ -643,8 +639,8 @@ namespace OpenBabel
 
   AtomExpr *OBSmartsPattern::ParseComplexAtomPrimitive( void )
   {
-    register Pattern *pat;
-    register int index;
+    Pattern *pat;
+    int index;
 
     switch( *LexPtr++ )
       {
@@ -1163,9 +1159,9 @@ namespace OpenBabel
 
   AtomExpr *OBSmartsPattern::ParseAtomExpr( int level )
   {
-    register AtomExpr *expr1 = NULL;
-    register AtomExpr *expr2 = NULL;
-    register char *prev;
+    AtomExpr *expr1 = NULL;
+    AtomExpr *expr2 = NULL;
+    char *prev;
 
     switch( level )
       {
@@ -1263,9 +1259,9 @@ namespace OpenBabel
 
   BondExpr *OBSmartsPattern::ParseBondExpr( int level )
   {
-    register BondExpr *expr1 = NULL;
-    register BondExpr *expr2 = NULL;
-    register char *prev;
+    BondExpr *expr1 = NULL;
+    BondExpr *expr2 = NULL;
+    char *prev;
 
     switch( level )
       {
@@ -1364,9 +1360,9 @@ namespace OpenBabel
                                 int prev, int part )
   {
     int vb = 0;
-    register AtomExpr *aexpr;
-    register BondExpr *bexpr;
-    register int index;
+    AtomExpr *aexpr;
+    BondExpr *bexpr;
+    int index;
 
     bexpr = (BondExpr*)0;
 
@@ -1486,7 +1482,7 @@ namespace OpenBabel
 
                 CreateBond(pat,bexpr,prev,stat->closure[index]);
                 pat->atom[prev].nbrs.push_back(stat->closure[index]);
-                for (int nbr_idx=0; nbr_idx < pat->atom[stat->closure[index]].nbrs.size(); ++nbr_idx) {
+                for (unsigned int nbr_idx=0; nbr_idx < pat->atom[stat->closure[index]].nbrs.size(); ++nbr_idx) {
                   if (pat->atom[stat->closure[index]].nbrs[nbr_idx] == -index)
                     pat->atom[stat->closure[index]].nbrs[nbr_idx] = prev;
                 }
@@ -1679,7 +1675,7 @@ namespace OpenBabel
 
   Pattern *OBSmartsPattern::ParseSMARTSString( char *ptr )
   {
-    register Pattern *result;
+    Pattern *result;
 
     if( !ptr || !*ptr )
       return (Pattern*)0;
@@ -1693,7 +1689,7 @@ namespace OpenBabel
 
   Pattern *OBSmartsPattern::ParseSMARTSRecord( char *ptr )
   {
-    register char *src;
+    char *src;
 
     src = ptr;
     while( *src && !isspace(*src) )
@@ -1709,56 +1705,19 @@ namespace OpenBabel
     return ParseSMARTSString(ptr);
   }
 
-  /*================================*/
-  /*  SMARTS Pattern simplification */
-  /*================================*/
-
-  static AtomExpr *NotAtomExpr( AtomExpr *expr )
-  {
-    register AtomExpr *result;
-
-    switch (expr->type)
-      {
-      case AE_TRUE:
-        expr->type = AE_FALSE;
-        return expr;
-      case AE_FALSE:
-        expr->type = AE_TRUE;
-        return expr;
-      case AE_AROMATIC:
-        expr->type = AE_ALIPHATIC;
-        return expr;
-      case AE_ALIPHATIC:
-        expr->type = AE_AROMATIC;
-        return expr;
-      case AE_CYCLIC:
-        expr->type = AE_ACYCLIC;
-        return expr;
-      case AE_ACYCLIC:
-        expr->type = AE_CYCLIC;
-        return expr;
-      
-      case AE_NOT:
-        result = expr->mon.arg;
-        expr->mon.arg = (AtomExpr*)0;
-        FreeAtomExpr(expr);
-        return result;
-      }
-    return BuildAtomNot(expr);
-  }
-
-
   //**********************************
   //********Pattern Matching**********
   //**********************************
 
   bool OBSmartsPattern::Init(const char *buffer)
   {
-	  if (_buffer != NULL)
-		  delete[] _buffer;
-	  _buffer = new char[strlen(buffer) + 1];
+    if (_buffer != NULL)
+      delete[] _buffer;
+    _buffer = new char[strlen(buffer) + 1];
     strcpy(_buffer,buffer);
 
+    if (_pat != NULL)
+      FreePattern(_pat);
     _pat = ParseSMARTSRecord(_buffer);
     _str = _buffer;
 
@@ -1767,15 +1726,17 @@ namespace OpenBabel
 
   bool OBSmartsPattern::Init(const std::string &s)
   {
-	if (_buffer != NULL)
-		delete[] _buffer;
-	_buffer = new char[s.length() + 1];
-	strcpy(_buffer, s.c_str());
+    if (_buffer != NULL)
+      delete[] _buffer;
+    _buffer = new char[s.length() + 1];
+    strcpy(_buffer, s.c_str());
 
-	_pat = ParseSMARTSRecord(_buffer);
-	_str = s;
+    if (_pat != NULL)
+      FreePattern(_pat);
+    _pat = ParseSMARTSRecord(_buffer);
+    _str = s;
 
-	return (_pat != (Pattern*) NULL);
+    return (_pat != (Pattern*) NULL);
   }
 
   OBSmartsPattern::~OBSmartsPattern()
@@ -2174,22 +2135,21 @@ namespace OpenBabel
                  !atom->IsAromatic();
         case AE_HCOUNT:
           return expr->leaf.value == ((int)atom->ExplicitHydrogenCount() +
-                                      (int)atom->ImplicitHydrogenCount());
+                                      (int)atom->GetImplicitHCount());
         case AE_CHARGE:
           return expr->leaf.value == atom->GetFormalCharge();
         case AE_CONNECT:
-          return expr->leaf.value == (int)atom->GetImplicitValence();
+          return expr->leaf.value == (int)atom->GetTotalDegree();
         case AE_DEGREE:
-          return expr->leaf.value == (int)atom->GetValence();
+          return expr->leaf.value == (int)atom->GetExplicitDegree();
         case AE_IMPLICIT:
-          return expr->leaf.value == (int)atom->ImplicitHydrogenCount();
+          return expr->leaf.value == (int)atom->GetImplicitHCount();
         case AE_RINGS:
           return expr->leaf.value == (int)atom->MemberOfRingCount();
         case AE_SIZE:
           return atom->IsInRingSize(expr->leaf.value);
         case AE_VALENCE:
-          return expr->leaf.value == (int)(atom->KBOSum()
-            - (atom->GetSpinMultiplicity() ? atom->GetSpinMultiplicity()-1 : 0));
+          return expr->leaf.value == (int)atom->GetTotalValence();
         case AE_CHIRAL:
           // always return true (i.e. accept the match) and check later
           return true;
@@ -2265,27 +2225,27 @@ namespace OpenBabel
         case BE_ANY:
           return true;
         case BE_DEFAULT:
-          return bond->GetBO()==1 || bond->IsAromatic();
+          return bond->GetBondOrder()==1 || bond->IsAromatic();
         case BE_SINGLE:
-          return bond->GetBO()==1 && !bond->IsAromatic();
+          return bond->GetBondOrder()==1 && !bond->IsAromatic();
         case BE_DOUBLE:
-          return bond->GetBO()==2 && !bond->IsAromatic();
+          return bond->GetBondOrder()==2 && !bond->IsAromatic();
         case BE_TRIPLE:
-          return bond->GetBO() == 3;
+          return bond->GetBondOrder() == 3;
         case BE_QUAD:
-          return bond->GetBO() == 4;
+          return bond->GetBondOrder() == 4;
         case BE_AROM:
           return bond->IsAromatic();
         case BE_RING:
           return bond->IsInRing();
-        case BE_UP:
-          return bond->IsUp();
-        case BE_DOWN:
-          return bond->IsDown();
-        case BE_UPUNSPEC: // up or unspecified (i.e., not down)
-          return !bond->IsDown();
-        case BE_DOWNUNSPEC: // down or unspecified (i.e., not up)
-          return !bond->IsUp();
+        //case BE_UP:
+        //  return bond->IsUp();
+        //case BE_DOWN:
+        //  return bond->IsDown();
+        //case BE_UPUNSPEC: // up or unspecified (i.e., not down)
+        //  return !bond->IsDown();
+        //case BE_DOWNUNSPEC: // down or unspecified (i.e., not up)
+        //  return !bond->IsUp();
         default:
           return false;
         }
