@@ -28,6 +28,8 @@ GNU General Public License for more details.
 
 #include <string>
 #include <math.h>
+#include <cstring>
+#include <algorithm>
 
 #ifndef M_PI
 #define M_PI 3.1415926535897932384626433832795028841971694
