@@ -89,7 +89,7 @@ namespace OpenBabel
   };
 
   /* Avoid SGI Compiler Warnings! */
-  char ElemDesc[MAXELEM][4] = {
+    char ElemDesc[MAXELEM][4] = {
     { ' ', 'N', ' ', ' ' },  /* 0*/
     { ' ', 'C', 'A', ' ' },  /* 1*/
     { ' ', 'C', ' ', ' ' },  /* 2*/
@@ -121,9 +121,6 @@ namespace OpenBabel
     { ' ', 'O', '6', ' ' }   /*28*/   /* 21-28 Nucleic Acid H-Bonding */
   };
 
-  unsigned int ResNo  = MINRES;
-  unsigned int ElemNo = MINELEM;
-
  /** \class OBResidue residue.h <openbabel/residue.h>
       \brief Residue information
 
@@ -329,30 +326,13 @@ namespace OpenBabel
               }
           }
 
-        unsigned int refno;
-        for( refno = MINELEM ; refno < ElemNo ; refno++ )
-          if( !strncmp(ElemDesc[refno], atomid, 4) )
-            return refno;
-
-        if ( ElemNo < MAXELEM - 1 )
-          {
-            ElemNo++;
-            ElemDesc[refno][0] = (char) ch1;
-            ElemDesc[refno][1] = (char) ch2;
-            ElemDesc[refno][2] = (char) ch3;
-            ElemDesc[refno][3] = (char) ch4;
-            return refno;
-          }
-        else
-          {
-            obErrorLog.ThrowError(__FUNCTION__, "Maximum number of atom ids exceeded", obWarning);
-            return 0;
-          }
+        return MAXELEM;
       }
+
     else
       {
         obErrorLog.ThrowError(__FUNCTION__, "NULL Atom IDs specified", obWarning);
-        return 0;
+        return MAXELEM;
       }
   }
 
@@ -751,19 +731,6 @@ namespace OpenBabel
             break;
           }
 
-        unsigned int refno;
-        for( refno = MINRES; refno < ResNo ; refno++ )
-          if( !strncmp(Residue[refno],res,3) )
-            return refno;
-
-        if ( ResNo < MAXRES - 1 )
-          {
-            ResNo++;
-            Residue[refno][0] = (char) ch1;
-            Residue[refno][1] = (char) ch2;
-            Residue[refno][2] = (char) ch3;
-            return refno;
-          }
       }
 
     return OBResidueIndex::UNK;
@@ -860,7 +827,7 @@ namespace OpenBabel
     _resnum   = "";
     _resname  = "";
     _vdata.clear();
-_insertioncode=0;
+    _insertioncode=0;
   }
 
   OBResidue::OBResidue(const OBResidue &src) :
@@ -874,7 +841,7 @@ _insertioncode=0;
     _atomid   = src._atomid;
     _hetatm   = src._hetatm;
     _sernum   = src._sernum;
-_insertioncode=src._insertioncode;
+    _insertioncode=src._insertioncode;
 
   }
 
@@ -905,7 +872,7 @@ _insertioncode=src._insertioncode;
         _atomid   = src._atomid;
         _hetatm   = src._hetatm;
         _sernum   = src._sernum;
-   _insertioncode = src._insertioncode;
+        _insertioncode = src._insertioncode;
       }
 
     return(*this);
@@ -1046,7 +1013,7 @@ _insertioncode=src._insertioncode;
         vector<OBBond*>::iterator b;
         for (bond = atom->BeginBond(b) ; bond ; bond = atom->NextBond(b))
           {
-            if (!idxs.BitIsOn(bond->GetIdx()))
+            if (!idxs.BitIsSet(bond->GetIdx()))
               {
                 if (!exterior)
                   {
@@ -1074,7 +1041,7 @@ _insertioncode=src._insertioncode;
     return _resnum;
   }
 
-  unsigned int OBResidue::GetNum(void)
+  int OBResidue::GetNum(void)
   {
     return atoi(_resnum.c_str());
   }
