@@ -1,6 +1,8 @@
 #include "obtest.h"
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
+#include <openbabel/atom.h>
+#include <openbabel/obiter.h>
 
 #include <iostream>
 #include <vector>
